#========================================================================================
# package      DE_EPAGES::Ebay::DAL::SQL::Categories
# state        private
# description  standard SQL database interface for ebay categories
#========================================================================================
package DE_EPAGES::Ebay::DAL::SQL::Categories;

use strict;
use DE_EPAGES::Core::API::PerlTools qw (GetLoopFromDoubleArray);

#========================================================================================
# function     GetRecentCategories
#----------------------------------------------------------------------------------------
# syntax       $rahRecentCategories = $Package->GetRecentCategories($dbi,$ShopID,$EnvID);
#----------------------------------------------------------------------------------------
# description  return all recently used categories order by usuage
#----------------------------------------------------------------------------------------
# input        $dbh | database handle | object
# input        $ShopID | Shop ID | integer
# input        $EnvID | Environment ID | integer
# input        $hParams | hash for additional parameters (optional) | object
# return       $rahRecentCategories | with following keys:
#               <ul>
#                   <li>EbayCategoryID  - Ebay Category id  - int
#               </ul>| reference to array of hashes
#========================================================================================
sub GetRecentCategories {
    my $self = shift;
    my ($dbi, $ShopID, $EnvID, $hParams) = @_;
    my $SqlStatement;
    my $aBindTypes;
    my $aBindVariables;

    if (defined $hParams and $hParams->{'IsUnityRegistered'}) {
        $aBindTypes = ['Integer','Integer','Integer','Integer'];
        $aBindVariables = [$ShopID, $EnvID, $ShopID, $EnvID];

        $SqlStatement = 'select ebaycategoryid from ebayauction ea '.
            'INNER JOIN ebayshopenvironment ese ON ea.ebayshopenvironmentid = ese.ebayshopenvironmentid '.
            'where ea.shopid=? and ese.ebayenvironmentid=? and ebaycategoryid > 0 '.
            'group by ebaycategoryid '.
            'UNION '.
            'select secondcategoryid from ebayauction ea '.
            'INNER JOIN ebayshopenvironment ese ON ea.ebayshopenvironmentid = ese.ebayshopenvironmentid '.
            'where ea.shopid=? and ese.ebayenvironmentid=? and secondcategoryid > 0 '.
            'group by secondcategoryid order by 1';


    } else {
        $aBindTypes = ['Integer','Integer'];
        $aBindVariables = [$ShopID, $EnvID];

        $SqlStatement = 'select ebaycategoryid from ebayauction ea '.
            'INNER JOIN ebayshopenvironment ese ON  ea.ebayshopenvironmentid = ese.ebayshopenvironmentid '.
            'where ea.shopid=? and ese.ebayenvironmentid=? and ebaycategoryid > 0 '.
            'group by ebaycategoryid order by count(auctionid) desc';
    }

    my $dbResult = $dbi->execute($SqlStatement, $aBindVariables, $aBindTypes);

    return GetLoopFromDoubleArray($dbResult, [qw (EbayCategoryID)]);
}

1;
