#========================================================================================
# package      DE_EPAGES::Klarna::API::KlarnaOperations::ReserveAmount
# base         DE_EPAGES::Klarna::API::KlarnaOperations::BaseOperation
# state        public
# description  Class responsible for gathering, and organazing, the data, and calling reserve amount.
#========================================================================================
package DE_EPAGES::Klarna::API::KlarnaOperations::ReserveAmount;
use base DE_EPAGES::Klarna::API::KlarnaOperations::BaseOperation;
use strict;

use DE_EPAGES::Core::API::Object::NumberFormat;
use DE_EPAGES::Klarna::API::Constants qw ( KLARNA_VERSION );

sub new {
    my $class = shift;
    my ($hParams, $LWPUserAgent)  = @_;
    my $self = __PACKAGE__->SUPER::new($LWPUserAgent);
    $self->{'Params'} = $hParams;
    return $self;
}

sub call {
    my $self = shift;
    $self->initialize();
    $self->_getParameters();

    $self->setInputData($self->{'KlarnaVersion'},   'string');
    $self->setInputData($self->{'client_vsn'},      'string');
    $self->setInputData($self->{'SecurityNumber'},  'string');
    $self->setInputData($self->{'gender'},          'int');
    $self->setInputData($self->{'amount'},          'int');
    $self->setInputData($self->{'reference'},       'string');
    $self->setInputData($self->{'reference_code'},  'string');
    $self->setInputData($self->{'orderid1'},        'string');
    $self->setInputData($self->{'orderid2'},        'string');
    $self->setInputData($self->{'delivery_addr'},   'struct');
    $self->setInputData($self->{'billing_addr'},    'struct');
    $self->setInputData($self->{'client_ip'},       'string');
    $self->setInputData($self->{'flags'},           'int');
    $self->setInputData($self->{'currency'},        'int');
    $self->setInputData($self->{'country'},         'int');
    $self->setInputData($self->{'language'},        'int');
    $self->setInputData($self->{'eid'},             'int');
    $self->setInputData($self->{'secret'},          'string');
    $self->setInputData($self->{'pno_encoding'},    'int');
    $self->setInputData($self->{'pclass'},          'int');
    $self->setInputData($self->{'goodsList'});
    $self->setInputData($self->{'comment'},         'string');
    $self->setInputData($self->{'shipmentinfo'},    'struct');
    $self->setInputData($self->{'travel_info'});
    $self->setInputData($self->{'income_expense'});
    $self->setInputData($self->{'bank_info'});
    $self->setInputData($self->{'session_id'});
    $self->setInputData($self->{'extra_info'});
    my $Response = $self->sendXML($self->{'IsInTestModeLineItemPayment'},
                        'reserve_amount',
                        $self->getInputData(),
                        $self->{PaymentTypeName});


    return ($self->handleReturn($Response));
}

sub _getParameters {
    my $self = shift;
    my $hParams = $self->{'Params'};
    my $hBasketOrderInfo = $hParams->{'BasketOrder'}->get( [qw(BillingAddress LineItemContainer Alias ObjectID)]);
    my $hLineItemContainerInfo = $hBasketOrderInfo->{LineItemContainer}->get( [qw(Payment GrandTotal CurrencyID)]);
    my $hPaymentMethodInfo = $hParams->{PaymentMethod}->get([qw(IsInTestMode Kr_StoreID Kr_SecretKey)]);
    my $hLineItemPaymentInfo = $hLineItemContainerInfo->{Payment}->get([qw(PaymentType)]);
    $self->{'IsInTestModeLineItemPayment'} = $hLineItemContainerInfo->{Payment}->get('IsTestTransaction');

    $self->{PaymentMethod} = $hParams->{PaymentMethod};
    my $DestinationAddress = $self->getAddress($hBasketOrderInfo->{BillingAddress},
                                                $hLineItemContainerInfo->{Payment},
                                                $hParams->{'Street'},
                                                $hParams->{'HouseNo'},
                                                $hParams->{'HouseNoExt'});

    $self->{PaymentTypeName} = $hLineItemPaymentInfo->{PaymentType}->alias;

    my $KlarnaFlags = 0;
    if ( $hPaymentMethodInfo->{IsInTestMode} &&
        $hPaymentMethodInfo->{Kr_StoreID} ne '611') #Special condition to test Klarna Invoice #todo:constans
    {
        $KlarnaFlags = 2;  #Test Mode #todo constans
    }

    my @ProductList = $self->getOrderLines($hBasketOrderInfo->{LineItemContainer},$self->{PaymentTypeName});

    $self->{'KlarnaVersion'} = KLARNA_VERSION;
    $self->{'client_vsn'} = $self->getClientVsn();
    $self->{'SecurityNumber'} = $hParams->{'SecurityNumber'};
    $self->{'gender'} = $self->getGender($hBasketOrderInfo->{BillingAddress});
    $self->{'amount'} = $self->_changePriceToInteger($hLineItemContainerInfo->{GrandTotal},$hLineItemContainerInfo->{CurrencyID});
    $self->{'reference'} = '';
    $self->{'reference_code'} = '';
    $self->{'orderid1'} = ($hParams->{'BasketOrder'}->instanceOf('CustomerOrder')) ? $hBasketOrderInfo->{'Alias'} : $hBasketOrderInfo->{'ObjectID'};
    $self->{'orderid2'} = '';
    $self->{'delivery_addr'} = $DestinationAddress;
    $self->{'billing_addr'} = $DestinationAddress;
    $self->{'client_ip'} = $hParams->{'RemoteAddr'};
    $self->{'flags'} = $KlarnaFlags;
    $self->{'currency'} = $self->getCurrency($hLineItemContainerInfo->{CurrencyID});
    $self->{'country'} = $self->getCountry($hBasketOrderInfo->{BillingAddress});
    $self->{'language'} = $self->getLanguage($hBasketOrderInfo->{BillingAddress});

    # Gets Integer value of Kr_StoreID. Solves AD-10018
    $self->{'eid'} = $hPaymentMethodInfo->{Kr_StoreID} + 0;

    my $aItemsForCheckSum = [$self->{'eid'},$self->{'SecurityNumber'},$self->{'amount'},$hPaymentMethodInfo->{Kr_SecretKey}];
    my $Digest = $self->buildDigest($aItemsForCheckSum);
    $self->{'secret'} = $Digest;
    $self->{'pno_encoding'} = $self->getEncoding($hBasketOrderInfo->{BillingAddress});
    $self->{'pclass'} = $self->getPclass($hBasketOrderInfo->{LineItemContainer});
    $self->{'goodsList'} = $self->serializeArray(\@ProductList);
    $self->{'comment'} = '';
    @{$self->{'shipmentinfo'}} = $self->getShipmentInfo();
    $self->{'travel_info'} = $self->serializeArray(());
    $self->{'income_expense'} = $self->serializeArray(());
    $self->{'bank_info'} = $self->serializeArray(());
    $self->{'session_id'} = $self->serializeArray(());
    $self->{'extra_info'} = $self->serializeArray(());
    return;
}

#========================================================================================
# function     _changePriceToInteger
# state        _private
#----------------------------------------------------------------------------------------
# syntax       _changePriceToInteger( $GrandTotal );
#----------------------------------------------------------------------------------------
# description  remove the comma from the float price. It will round the price to 2 digits
#----------------------------------------------------------------------------------------
# input        $Price | a ePages Price | float
# input        $CurrencyID | the currency ID (like: EUR) | String
# return       $PriceAsInteger | price as integer | integer
#========================================================================================
sub _changePriceToInteger {
    my $self = shift;
    my ($Price, $CurrencyID) = @_;

    my $PriceAsInteger;
    my $NumberFormater = DE_EPAGES::Core::API::Object::NumberFormat->new(('int_currencyid' => $CurrencyID));
    ## no critic (ProhibitMagicNumbers)
    my $decimalDigitsForKlarna = 2;
    $Price = $NumberFormater->roundTotal($Price, $decimalDigitsForKlarna);
    my $Factor = 10 ** $decimalDigitsForKlarna;
    $PriceAsInteger = int( $Price * $Factor );
    ## use critic (ProhibitMagicNumbers)
    return $PriceAsInteger;
}

1;