#========================================================================================
# package      DE_EPAGES::Saferpay2::UI::PaymentMethodSaferpay2
# base         DE_EPAGES::Payment::UI::PaymentMethod
# state        private
#----------------------------------------------------------------------------------------
# description  UI Controller for payment method
#========================================================================================
package DE_EPAGES::Saferpay2::UI::PaymentMethodSaferpay2;
use base DE_EPAGES::Payment::UI::PaymentMethod;
use DE_EPAGES::Saferpay2::API::SaferpayService;
use DE_EPAGES::Core::API::PerlTools qw (isValid);
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Saferpay2::API::Constants qw ( PAYMENT_TYPE_SAFERPAY2 );
use constant ERROR_TRANSACTION_NOT_FOUND => 'TRANSACTION_NOT_FOUND';

use strict;



#========================================================================================
# function     ViewSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to view settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub ViewSettings {
    my $self = shift;
    my ($Servlet) = @_;

    return;
}



#========================================================================================
# function     SaveSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       SaveSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Controller to save settings on the MBO settings page.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::BasePermission
#========================================================================================
sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $PaymentMethod = $Servlet->object();
    my $Form = $Servlet->form();

    my $hFormValues = $Form->form($PaymentMethod, 'Saferpay2Settings');

    $Servlet->vars('GetMerchantStatusError', 0);
    unless ($self->_checkCredentials($hFormValues)) {
        $Servlet->vars('GetMerchantStatusError', 1);
        $Form->addFormError({
            'Form' => 'Saferpay2Settings',
            'Reason' => 'CredentialsValidationError',
            'Name' => 'CredentialsValidationError' });
        $Form->executeFormError();
        return;
    }

    $self->Save($Servlet);
}

sub _checkCredentials {
    my $self = shift;
    my ($hFormValues) = @_;

    my $hValues = {
        MerchantName => $hFormValues->{'MerchantName'},
        MerchantPassword => $hFormValues->{'MerchantPassword'},
        CustomerId => $hFormValues->{'CustomerId'},
        TerminalId => $hFormValues->{'TerminalId'},
        IsInTestMode => $hFormValues->{'IsInTestMode'},
    };

    my $SaferpayService = DE_EPAGES::Saferpay2::API::SaferpayService->new(undef, $hValues);
    $self->_LogPayment('DoAssertCall Request to validate the credentials.', { 'MerchantName' => $hFormValues->{'MerchantName'} });
    my $hResponse = $SaferpayService->DoAssertCall("ValidateCredentialsInvalidToken");
    my $CredentialsChecked = (isValid($hResponse->{'ErrorName'}) &&
        $hResponse->{'ErrorName'} eq ERROR_TRANSACTION_NOT_FOUND) ? 1 : 0;

    return $CredentialsChecked;
}


sub _LogPayment {
    my $self = shift;
    my ($Name, $DebugData, $Section, $Filter) = @_;

    return LogPayment(PAYMENT_TYPE_SAFERPAY2, $Name, $DebugData, $Section, $Filter);
}
1;